<?php
/**
 * Perforce Swarm
 *
 * @copyright   2013-2020 Perforce Software. All rights reserved.
 * @license     Please see LICENSE.txt in top-level readme folder of this distribution.
 * @version     2019.3/2027974
 */
namespace TestIntegration\Filter;

use Application\Filter\ArrayValues;
use Application\I18n\TranslatorFactory;
use Application\InputFilter\DirectInput;
use Application\InputFilter\InputFilter;
use Application\Validator\FlatArray;
use Interop\Container\ContainerInterface;
use TestIntegration\Model\TestDefinition as Model;
use Zend\Filter\StringTrim;
use Zend\InputFilter\Input;
use Zend\Validator\GreaterThan;
use Zend\Validator\Regex;
use Zend\Validator\StringLength;

/**
 * Class TestDefinitionFilter to filter and validate TestDefinition
 * @package TestIntegration\Filter
 */
class TestDefinition extends InputFilter implements ITestDefinition
{
    private $translator;
    private $services;

    /**
     * @inheritDoc
     */
    public function __construct(ContainerInterface $services, array $options = null)
    {
        $this->services   = $services;
        $this->translator = $services->get(TranslatorFactory::SERVICE);
        $this->addHeaderFilter();
        $this->addTitleFilter();
        $this->addEncodingFilter();
        $this->addNameFilter();
        $this->addBodyFilter();
        $this->addUrlFilter();
        $this->addTimeoutFilter();
    }

    /**
     * Add to the chain to filter and validate encoding
     */
    private function addEncodingFilter()
    {
        $input = new Input(Model::FIELD_ENCODING);
        $input->getFilterChain()->attach(new StringTrim());
        $input->getValidatorChain()->attach(new StringLength(['min' => 3, 'max' => 4]));
        $input->getValidatorChain()->attach(new EncodingValidator($this->translator));
        $this->add($input);
    }

    /**
     * Add to the chain to filter and validate headers
     */
    private function addHeaderFilter()
    {
        $input = new Input(Model::FIELD_HEADERS);
        $input->setRequired(false);
        $input->getFilterChain()->attach(new ArrayValues);
        $input->getValidatorChain()->attach(new FlatArray);
        $this->add($input);
    }

    /**
     * Add to the chain to filter and validate timeout
     */
    private function addTimeoutFilter()
    {
        $input = new Input(Model::FIELD_TIMEOUT);
        $input->setRequired(false);
        $input->getValidatorChain()->attach(new GreaterThan(['min' => 0]));
        $this->add($input);
    }

    /**
     * Add to the chain to filter and validate body
     */
    private function addBodyFilter()
    {
        $input = new DirectInput(Model::FIELD_BODY);
        $input->setRequired(false);
        $input->getFilterChain()->attach(new StringTrim());
        $input->getValidatorChain()
            ->attach(new StringLength(['min' => 1]))
            ->attach(new BodyValidator($this->services));
        $this->add($input);
    }

    /**
     * Add to the chain to filter and validate url
     */
    private function addUrlFilter()
    {
        $input = new Input(Model::FIELD_URL);
        $input->getFilterChain()->attach(new StringTrim());
        $input->getValidatorChain()->attach(new StringLength(['min' => 1, 'max' => 1024]));
        $this->add($input);
    }

    /**
     * Add to the chain to filter and validate title
     */
    private function addTitleFilter()
    {
        $input = new Input(Model::FIELD_TITLE);
        $input->getFilterChain()->attach(new StringTrim());
        $input->getValidatorChain()->attach(new StringLength(['min' => 1, 'max' => 64]));
        $this->add($input);
    }

    /**
     * Add to the chain to filter and validate title
     */
    private function addNameFilter()
    {
        $input = new Input(Model::FIELD_NAME);
        $input->getFilterChain()->attach(new StringTrim());
        $input->getValidatorChain()->attach(new NamesValidator($this->services));
        $input->getValidatorChain()->attach(new Regex(['pattern' => "/^[-_A-Za-z0-9]/"]));
        $input->getValidatorChain()->attach(new StringLength(['min' => 1, 'max' => 32]));
        $this->add($input);
    }
}
